from maemofuse.fs import loader as fsloader

from maemofuse.sshfs import SSHFs
from maemofuse.flickrfs import FlickrFS

import gobject
import gtk
import gtk.glade
import hildon
import osso

import os.path
import pickle

#gladefile = 'maemofuse.glade'
gladefile = '/usr/share/fuse/maemofuse.glade'


class Gui:
    def __init__ (self, context):
        self.window = hildon.Window ()
        self.window.connect ("destroy", gtk.main_quit)
        self.xml = gtk.glade.XML (gladefile, 'hbox1')
        self.xml_add = gtk.glade.XML (gladefile, 'vbox2')
        self.xml.signal_autoconnect (self)
        self.xml_add.signal_autoconnect (self)
        
        self.context = context
        
        mbox = self.xml.get_widget ('hbox1')
        self.window.add (mbox)

        self.wadd = hildon.Window ()
        self.comboadd = gtk.combo_box_new_text ()
        self.comboadd.connect ('changed', self.combo_changed)
        for f in fsloader.fs:
            self.comboadd.append_text (f.name)
        self.comboadd.set_active (0)
        box = self.xml_add.get_widget ('vbox2')        
        hbox = self.xml_add.get_widget ('hbox3')
        hbox.pack_start (self.comboadd, False, False)
        self.wadd.add (box)    
        self.tree = self.xml.get_widget ('treeview1')
        
        #create treeview        
        self.data_list = gtk.ListStore (gobject.TYPE_PYOBJECT, gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.tree.set_model (self.data_list)
        sel = self.tree.get_selection ()
        sel.set_mode (gtk.SELECTION_MULTIPLE)

        tvcol = gtk.TreeViewColumn('')
        tvcol.set_visible = False
        self.tree.append_column (tvcol)
        cell = gtk.CellRendererText()
        tvcol.pack_start(cell, True)
        
        tvcol = gtk.TreeViewColumn('Mount Point')
        self.tree.append_column (tvcol)
        cell = gtk.CellRendererText()
        tvcol.pack_start(cell, True)
        tvcol.add_attribute(cell, 'text', 1)

        tvcol = gtk.TreeViewColumn('Type')
        self.tree.append_column (tvcol)
        cell = gtk.CellRendererText()
        tvcol.pack_start(cell, True)
        tvcol.add_attribute(cell, 'text', 2)
        
        #populate treeview
        self.data = self.load_data ()        
        self.reload_fs_list ()
    
    def reload_fs_list (self):
        self.data_list.clear ()
        for fs in self.data:
            opts = fs.get_options ()
            mount = opts ['mountpoint'].value
            name = fs.name
            self.data_list.append ([fs, mount, name])
        
    
    def load_data (self):
        #load data from home directory..        
        try:
            f = open (os.path.expanduser ('~/.maemofuse'))
            data = pickle.load (f)
            f.close ()
            return data
        except Exception, e:
            print "Error loading data..", e
            return []  
    
    def write_data (self):
        f = open (os.path.expanduser ('~/.maemofuse'), 'w')
        pickle.dump (self.data, f)
        f.close ()
    
    def combo_changed (self, combo):
        self.reload_combo_add ()
    
    def show (self):
        self.window.show_all ()
        
    def combo_remove (self, wig, vbox):
        vbox.remove (wig)

    def button_browse_mountpoint_clicked (self, btn, o):
        dlg = hildon.FileChooserDialog (self.wadd, gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        res = dlg.run()
        if res == gtk.RESPONSE_OK:
            value = dlg.get_filename ()
            print value
            o.wig.set_text (value)
        dlg.destroy ()

    def reload_combo_add (self):
        ret = self.comboadd.get_active_text ()
        print "reload", ret
        fs = fsloader [ret].obj['new'] ()
        self.current_fs = fs
        vbox = self.xml_add.get_widget ('vbox3')
        #clear all
        vbox.foreach (self.combo_remove, vbox)
        for o in fs.get_options ().values ():
            h = gtk.HBox ()
            l = gtk.Label (o.name + ':')
            t = gtk.Entry ()
            o.wig = t
            if o.value != None:
                t.set_text (o.value)
            if o.visible == False:
                o.wig.set_visibility (False)
            h.pack_start (l)
            h.pack_start (t)
            if o.fs == True:
                b = gtk.Button (label = "...")
                b.connect ('clicked', self.button_browse_mountpoint_clicked, o)
                h.pack_start (b)

            vbox.pack_start (h)
            #create an option and add it to the gui..
        vbox.show_all ()
    
    def on_buttonAdd_clicked (self, btn):
        self.reload_combo_add ()
        self.wadd.show_all ()
    
    def on_buttonOK_clicked (self, btn):
        fs = self.current_fs
        ret = self.comboadd.get_active_text ()        
        for o in fs.get_options ().values ():
            o.value = o.wig.get_text ()
            o.wig = None
        self.data.append (fs)
        self.write_data ()
        self.reload_fs_list ()
        self.wadd.hide ()
    
    def get_fs_selected (self):
        sel = self.tree.get_selection ()
        (model, rows) = sel.get_selected_rows ()
        ret = []
        for r in rows:
            fs = model.get_value (model.get_iter (r), 0)
            ret.append (fs)
        return ret
        
    def on_buttonDelete_clicked (self, btn):
        ret = self.get_fs_selected ()
        for fs in ret:
            self.data.remove (fs)
        self.write_data ()
        self.reload_fs_list ()
    
    def on_buttonMount_clicked (self, btn):
        ret = self.get_fs_selected ()
        for fs in ret:
            ret = fs.mount ()
            print "Mount", ret
            if ret[0]:
                self.context.system_note_infoprint ('Fs mounted without errors')
            else:
                self.context.system_note_dialog (ret[1], 'error')        

    def on_buttonUmount_clicked (self, btn):
        ret = self.get_fs_selected ()
        for fs in ret:
            ret = fs.umount ()
            print "Umount", ret
            if ret[0]:
                self.context.system_note_infoprint ('Fs umounted without errors')
            else:
                self.context.system_note_dialog (ret[1], 'error')        
    